@@ -7,7 +7,7 @@ from django.db.models import Q
 
 from ..compat import LOOKUP_SEP
 from ..exceptions import QueryablePropertyError
-from ..utils import InjectableMixin
+from ..utils.internal import InjectableMixin
 
 
 class LookupFilterMeta(type):
@@ -136,6 +136,17 @@ class AnnotationMixin(InjectableMixin):
 
     filter_requires_annotation = True
 
+    @property
+    def admin_order_field(self):
+        """
+        Return the field name for the ordering in the admin, which is simply
+        the property's name since it's annotatable.
+
+        :return: The field name for ordering in the admin.
+        :rtype: str
+        """
+        return self.name
+
     def get_annotation(self, cls):  # pragma: no cover
         """
         Construct an annotation representing this property that can be added
@@ -153,6 +164,54 @@ class AnnotationMixin(InjectableMixin):
         return Q(**{LOOKUP_SEP.join((self.name, lookup)): value})
 
 
+class AnnotationGetterMixin(AnnotationMixin):
+    """
+    A mixin for queryable properties that support annotation and use their
+    annotation even to provide the value for their getter (i.e. perform a query
+    to retrieve the getter value).
+    """
+
+    def __init__(self, cached=None, *args, **kwargs):
+        """
+        Initialize a new queryable property based that uses the
+        :class:`AnnotationGetterMixin`.
+
+        :param cached: Determines if values obtained by the getter should be
+                       cached (similar to ``cached_property``). A value of None
+                       means using the default value.
+        """
+        super(AnnotationGetterMixin, self).__init__(*args, **kwargs)
+        if cached is not None:
+            self.cached = cached
+
+    def get_value(self, obj):
+        queryset = self.get_queryset_for_object(obj).distinct()
+        queryset = queryset.annotate(**{self.name: self.get_annotation(obj.__class__)})
+        return queryset.values_list(self.name, flat=True).get()
+
+    def get_queryset(self, model):
+        """
+        Construct a base queryset for the given model class that can be used
+        to build queries in property code.
+
+        :param model: The model class to build the queryset for.
+        """
+        return model._base_manager.all()
+
+    def get_queryset_for_object(self, obj):
+        """
+        Construct a base queryset that can be used to retrieve the getter value
+        for the given object.
+
+        :param django.db.models.Model obj: The object to build the queryset
+                                           for.
+        :return: A base queryset for the correct model that is already filtered
+                 for the given object.
+        :rtype: django.db.models.QuerySet
+        """
+        return self.get_queryset(obj.__class__).filter(pk=obj.pk)
+
+
 class UpdateMixin(object):
     """
     A mixin for queryable properties that allows to use themselves in update
