@@ -1,6 +1,5 @@
 # encoding: utf-8
 
-from collections import namedtuple
 from contextlib import contextmanager
 from functools import partial
 
@@ -9,77 +8,11 @@ from django.utils.tree import Node
 
 from .compat import (
     ADD_Q_METHOD_NAME, ANNOTATION_TO_AGGREGATE_ATTRIBUTES_MAP, BUILD_FILTER_METHOD_NAME, contains_aggregate,
-    convert_build_filter_to_add_q_kwargs, get_related_model, LOOKUP_SEP, NEED_HAVING_METHOD_NAME, nullcontext,
-    QUERY_CHAIN_METHOD_NAME
+    convert_build_filter_to_add_q_kwargs, LOOKUP_SEP, NEED_HAVING_METHOD_NAME, nullcontext, QUERY_CHAIN_METHOD_NAME,
+    ValuesQuerySet
 )
-from .exceptions import FieldDoesNotExist, QueryablePropertyDoesNotExist, QueryablePropertyError
-from .utils import get_queryable_property, InjectableMixin, TreeNodeProcessor
-
-
-class QueryablePropertyReference(namedtuple('QueryablePropertyReference', 'property model relation_path')):
-    """
-    A reference to a queryable property that also holds the path to reach the
-    property across relations.
-    """
-    __slots__ = ()
-
-    @property
-    def full_path(self):
-        """
-        Return the full path to the queryable property (including the relation
-        prefix) in the query filter format.
-
-        :return: The full path to the queryable property.
-        :rtype: str
-        """
-        if not self.relation_path:
-            return self.property.name
-        return LOOKUP_SEP.join(self.relation_path + (self.property.name,))
-
-    def get_filter(self, lookups, value):
-        """
-        A wrapper for the get_filter method of the property this reference
-        points to. It checks if the property actually supports filtering and
-        applies the relation path (if any) to the returned Q object.
-
-        :param collections.Sequence[str] lookups: The lookups/transforms to use
-                                                  for the filter.
-        :param value: The value passed to the filter condition.
-        :return: A Q object to filter using this property.
-        :rtype: django.db.models.Q
-        """
-        if not self.property.get_filter:
-            raise QueryablePropertyError('Queryable property "{}" is supposed to be used as a filter but does not '
-                                         'implement filtering.'.format(self.property))
-
-        # Use the model stored on this reference instead of the one on the
-        # property since the query may be happening from a subclass of the
-        # model the property is defined on.
-        q_obj = self.property.get_filter(self.model, LOOKUP_SEP.join(lookups) or 'exact', value)
-        if self.relation_path:
-            # If the resolved property belongs to a related model, all actual
-            # conditions in the returned Q object must be modified to use the
-            # current relation path as prefix.
-            def prefix_condition(item):
-                return LOOKUP_SEP.join(self.relation_path + (item[0],)), item[1]
-            q_obj = TreeNodeProcessor(q_obj).modify_leaves(prefix_condition)
-        return q_obj
-
-    def get_annotation(self):
-        """
-        A wrapper for the get_annotation method of the property this reference
-        points to. It checks if the property actually supports annotation
-        creation performs the internal call with the correct model class.
-
-        :return: An annotation object.
-        """
-        if not self.property.get_annotation:
-            raise QueryablePropertyError('Queryable property "{}" needs to be added as annotation but does not '
-                                         'implement annotation creation.'.format(self.property))
-        # Use the model stored on this reference instead of the one on the
-        # property since the query may be happening from a subclass of the
-        # model the property is defined on.
-        return self.property.get_annotation(self.model)
+from .exceptions import QueryablePropertyError
+from .utils.internal import InjectableMixin, resolve_queryable_property, TreeNodeProcessor
 
 
 class QueryablePropertiesQueryMixin(InjectableMixin):
@@ -105,49 +38,6 @@ class QueryablePropertiesQueryMixin(InjectableMixin):
         # Required to correctly resolve dependencies and perform annotations.
         self._queryable_property_stack = []
 
-    def _resolve_queryable_property(self, path):
-        """
-        Resolve the given path into a queryable property on the model
-        associated with this query.
-
-        :param collections.Sequence[str] path: The path to resolve (a string of
-                                               Django's query expression split
-                                               up by the lookup separator).
-        :return: A 2-tuple containing a queryable property reference for the
-                 resolved property and a list containing the parts of the path
-                 that represent lookups (or transforms). The first item will be
-                 None and the list will be empty if no queryable property could
-                 be resolved.
-        :rtype: (QueryablePropertyReference, list[str])
-        """
-        model = self.model
-        property_ref, lookups = None, []
-        # Try to follow the given path to allow to use queryable properties
-        # across relations.
-        for index, name in enumerate(path):
-            try:
-                related_model = get_related_model(model, name)
-            except FieldDoesNotExist:
-                try:
-                    prop = get_queryable_property(model, name)
-                except QueryablePropertyDoesNotExist:
-                    # Neither a field nor a queryable property, so likely an
-                    # invalid name. Do nothing and let Django deal with it.
-                    pass
-                else:
-                    property_ref = QueryablePropertyReference(prop, model, tuple(path[:index]))
-                    lookups = path[index + 1:]
-                # The current name was not a field and either a queryable
-                # property or invalid. Either way, resolving ends here.
-                break
-            else:
-                if not related_model:
-                    # A regular model field that doesn't represent a relation,
-                    # meaning that no queryable property is involved.
-                    break
-                model = related_model
-        return property_ref, lookups
-
     @contextmanager
     def _add_queryable_property_annotation(self, property_ref, full_group_by, select=False):
         """
@@ -158,9 +48,9 @@ class QueryablePropertiesQueryMixin(InjectableMixin):
         manager yields the actual resolved and applied annotation while the
         stack is still populated.
 
-        :param QueryablePropertyReference property_ref: A reference containing
-                                                        the queryable property
-                                                        to annotate.
+        :param property_ref: A reference containing the queryable property
+                             to annotate.
+        :type property_ref: queryable_properties.utils.internal.QueryablePropertyReference
         :param bool full_group_by: Signals whether to use all fields of the
                                    query for the GROUP BY clause when dealing
                                    with an aggregate-based annotation or not.
@@ -202,7 +92,7 @@ class QueryablePropertiesQueryMixin(InjectableMixin):
                     self.add_fields([f.attname for f in getattr(opts, 'concrete_fields', opts.fields)], False)
                 self.set_group_by()
 
-    def _auto_annotate(self, path):
+    def _auto_annotate(self, path, full_group_by=None):
         """
         Try to resolve the given path into a queryable property and annotate
         the property as a non-selected property (if the property wasn't added
@@ -212,13 +102,20 @@ class QueryablePropertiesQueryMixin(InjectableMixin):
         :param collections.Sequence path: The path to resolve (a string of
                                           Django's query expression split up
                                           by the lookup separator).
+        :param bool | None full_group_by: Optional override to indicate whether
+                                          or not all fields must be contained
+                                          in a GROUP BY clause for aggregate
+                                          annotations. If not set, it will be
+                                          determined from the state of this
+                                          query.
         :return: The resolved annotation or None if the path couldn't be
                  resolved.
         """
-        property_ref = self._resolve_queryable_property(path)[0]
+        property_ref = resolve_queryable_property(self.model, path)[0]
         if not property_ref:
             return None
-        full_group_by = bool(ANNOTATION_TO_AGGREGATE_ATTRIBUTES_MAP) and not self.select
+        if full_group_by is None:
+            full_group_by = bool(ANNOTATION_TO_AGGREGATE_ATTRIBUTES_MAP) and not self.select
         with self._add_queryable_property_annotation(property_ref, full_group_by) as annotation:
             return annotation
 
@@ -298,7 +195,7 @@ class QueryablePropertiesQueryMixin(InjectableMixin):
             # and delegate it to Django.
             property_ref = None
         else:
-            property_ref, lookups = self._resolve_queryable_property(arg.split(LOOKUP_SEP))
+            property_ref, lookups = resolve_queryable_property(self.model, arg.split(LOOKUP_SEP))
 
         # If no queryable property could be determined for the filter
         # expression (either because a regular/non-existent field is referenced
@@ -355,7 +252,7 @@ class QueryablePropertiesQueryMixin(InjectableMixin):
         # return True if a filter condition contains such a property.
         def is_aggregate_property(item, ignored_refs=set()):
             path = item[0].split(LOOKUP_SEP)
-            property_ref, lookups = self._resolve_queryable_property(path)
+            property_ref, lookups = resolve_queryable_property(self.model, path)
             if not property_ref or property_ref in ignored_refs:
                 return False
             if property_ref.property.filter_requires_annotation:
@@ -384,7 +281,7 @@ class QueryablePropertiesQueryMixin(InjectableMixin):
         path = tuple(name.split(LOOKUP_SEP))
         if self._queryable_property_stack:
             path = self._queryable_property_stack[-1].relation_path + path
-        property_annotation = self._auto_annotate(path)
+        property_annotation = self._auto_annotate(path, full_group_by=ValuesQuerySet is not None)
         if property_annotation:
             if summarize:
                 # Outer queries for aggregations need refs to annotations of
