@@ -1,5 +1,7 @@
 # encoding: utf-8
 
+from django import VERSION as DJANGO_VERSION
+
 SECRET_KEY = '1337' * 8
 
 DATABASES = {
@@ -24,7 +26,9 @@ INSTALLED_APPS = (
     'django.contrib.admin',
     'django.contrib.admindocs',
     'queryable_properties',
-    'tests',
+    'tests.app_management',
 )
+if DJANGO_VERSION < (1, 9):
+    INSTALLED_APPS += ('tests.dummy_lib',)  # Django versions before 1.9 don't support abstract models outside of apps
 
 SITE_ID = 1
